/*
 -------------------------------------------------------------------------------
 | Producer - Copyright (C) 2001-2007  Don Burns                               |
 |                                                                             |
 | This library is free software; you can redistribute it and/or modify        |
 | it under the terms of the GNU Lesser General Public License as published    |
 | by the Free Software Foundation; either version 3 of the License, or        |
 | (at your option) any later version.                                         |
 |                                                                             |
 | This library is distributed in the hope that it will be useful, but         |
 | WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  |
 | or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public     |
 | License for more details.                                                   |
 |                                                                             |
 | You should have received a copy of the GNU Lesser General Public License    |
 | along with this software; if not, write to the Free Software Foundation,    |
 | Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.               |
 |                                                                             |
 -------------------------------------------------------------------------------
 */

#ifndef FLKT_RENDER_SURFACE_DEF
#define FLKT_RENDER_SURFACE_DEF 1

#include <Producer/RenderSurface.h>
#include <FL/Fl_Gl_Window.H>

class FLTKRenderSurface: public Producer::RenderSurface, public Fl_Gl_Window
{
    public:

        FLTKRenderSurface( int X, int Y, int W, int H, const char *L="" ):
            Fl_Gl_Window(X, Y, W, H, L),
            _mx(0),
            _my(0),
            _mbutton(0)

        {
        }

        virtual bool realize( Producer::VisualChooser *vc=NULL, Producer::GLContext sharedGLContext=0 ) 
        {
            return _realized = true;
        }

        virtual void getWindowRectangle( int &x, int &y, unsigned int &width, unsigned int &height ) const
        {
            x = this->x();
            y = this->y();
            width = this->w();
            height = this->h();
        }

        virtual bool makeCurrent( bool ) 
        {
            make_current();
            return true;
        }

        virtual void swapBuffers()
        {
            swap_buffers();
        }

        virtual void draw()
        {
            if( _drawCallback != NULL )
                (*_drawCallback)(this, _drawCallbackData );
        }

        virtual void setDrawCallback( Fl_Callback *callback, void *data )
        {
            _drawCallback = callback;
            _drawCallbackData = data;
        }

        int handle(int event)
        {
            switch(event)
            {
                case FL_PUSH : 
                    _mx = Fl::event_x();
                    _my = Fl::event_y();
                    _mbutton |= 1 << (Fl::event_button()-1);
                    redraw();
                    return 1;

                case FL_RELEASE:
                    _mx = Fl::event_x();
                    _my = Fl::event_y();
                    _mbutton &= ~(1 << (Fl::event_button()-1));
                    redraw();
                    return 1;

                case FL_DRAG :
                    _mx = Fl::event_x();
                    _my = Fl::event_y();
                    redraw();
                    return 1;


                case FL_FOCUS :
                    redraw();
                    return 1;

                default:
                // pass other events to the base class...
                return Fl_Gl_Window::handle(event);
            }
        }


    private:
            Fl_Callback *_drawCallback;
            void *_drawCallbackData;

    protected:
            // Button State
            int _mx, _my;
            unsigned int _mbutton;
};

#endif

